import * as process from 'process'
import * as https from 'https'
import axios, { AxiosHeaders, AxiosInstance } from 'axios'
import { VBRListAllJobStateResponseI } from '../interfaces/VBRResponse.Interface'
import Helper from '../utils/Helper'
import Logger from '../utils/Logger'
import * as moment from 'moment'

export default class VBRConnector {
  private readonly baseUrl = process.env.VBR_BASE_URL
  private readonly vbrUsername = process.env.VBR_USERNAME
  private readonly vbrPassword = process.env.VBR_PASSWORD
  private readonly axios: AxiosInstance
  private isManagerRun = false
  private accessToken = ''

  constructor() {
    this.axios = axios.create({
      baseURL: this.baseUrl,
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        'x-api-version': '1.1-rev0',
      },
      httpsAgent: new https.Agent({
        rejectUnauthorized: false,
      }),
    })

    this.setupAxiosInterceptor()
  }

  async setupAxiosInterceptor() {
    const retryErrorCode = [401, 403]

    this.axios.interceptors.request.use(
      (config) => {
        config.headers = new AxiosHeaders({
          ...config.headers,
          Authorization: `Bearer ${this.accessToken}`,
        })

        return config
      },
      (error) => {
        return Promise.reject(error)
      },
    )

    this.axios.interceptors.response.use(
      (response) => {
        return response
      },
      async (error) => {
        const originalRequest = error.config
        console.log(error.response.status)
        console.log(error.response.statusText)
        if (
          retryErrorCode.includes(error.response.status) &&
          !originalRequest._retry
        ) {
          originalRequest._retry = true

          await this.login()
          this.axios.defaults.headers.common[
            'Authorization'
          ] = `Bearer ${this.accessToken}`
          return this.axios(originalRequest)
        }
        return Promise.reject(error)
      },
    )
  }

  async ensureManagerRunning() {
    Logger.info(`Ensuring Veeam Backup Manager is up and running...`)

    if (this.isManagerRun === true) {
      return true
    }

    while (!this.isManagerRun) {
      const check = await this.checkManagerRunning()

      if (check) {
        this.isManagerRun = true
      } else {
        await Helper.sleep(30 * 1000)
      }
    }

    Logger.info(`Veeam Backup Manager is up and running.`)
    return true
  }

  async checkManagerRunning() {
    const notStartedCode = ['ETIMEDOUT', 'ECONNABORTED']
    try {
      await this.axios.get('v1/sessions', {
        timeout: 5000,
      })

      return true
    } catch (e) {
      if (notStartedCode.includes(e.code)) {
        return false
      }
    }
  }

  async login() {
    const formData = new URLSearchParams({
      grant_type: 'password',
      username: this.vbrUsername,
      password: this.vbrPassword,
      use_short_term_refresh: 'true',
    })

    const query = await this.axios.post('oauth2/token', formData.toString())

    this.accessToken = query.data.access_token
    return
  }
  async logout() {
    return this.axios.post('oauth2/logout')
  }

  async listBackupJobs() {
    const formData = new URLSearchParams({
      skip: '0',
      limit: '0',
      orderColumn: 'Name',
      orderAsc: 'true',
      typeFilter: 'Backup',
    })

    const query = await this.axios.get('v1/jobs', {
      data: formData.toString(),
    })
    console.log(query.data)
  }

  async listBackupJobStates() {
    console.log('called list backup job')
    const formData = new URLSearchParams({
      skip: '0',
      limit: '0',
      orderColumn: 'Name',
      orderAsc: 'true',
      typeFilter: 'Backup',
    })
    return this.axios.get<VBRListAllJobStateResponseI>('v1/jobs/states', {
      data: formData.toString(),
    })
  }

  async ensureAllBackupJobDone(dateNow: moment.Moment) {
    Logger.info(`Ensuring Veeam Backup Job is running successfully...`)

    let isBackupDone = false
    let tempState = false

    while (!isBackupDone) {
      tempState = true

      const query = await this.listBackupJobStates()
      query.data.data.forEach((job) => {
        if (job.status === 'disabled') {
          return
        }

        const nextRun = moment(job.nextRun)
        const lastRun = moment(job.lastRun)

        if (job.status === 'running') {
          tempState = false
        }

        if (nextRun <= dateNow) {
          tempState = false
        }

        if (lastRun <= dateNow) {
          tempState = false
        }
      })

      if (tempState) {
        isBackupDone = true
      } else {
        await Helper.sleep(60 * 1000)
      }
    }

    Logger.info(`Veeam Backup Job is running successfully.`)
    await this.logout()
    return true
  }
}
